# Copyright (c) 2003 Chalmers University of Technology
# Authors: Jean-Marc Orliaguet <jmo@ita.chalmers.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

"""
  Base Style
"""

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from Acquisition import aq_base, aq_parent
from Products.CMFCore.utils import _verifyActionPermissions
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.PortalContent import PortalContent

from Products.CPSSkins.PortalThemesTool import PortalThemesTool

def addBaseStyle(dispatcher, id, REQUEST=None):
    """Add a Base Style."""
    ob = BaseStyle(id)
    container = dispatcher.Destination()
    container._setObject(id, ob)
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)


factory_type_information = (
    {'id': 'Base Style',
     'description': ('A Base Style is the most basic style.'),
     'meta_type': 'Base Style',
     'icon': 'style_icon.gif',
     'product': 'CPSSkins',
     'factory': 'addBaseStyle',
     'immediate_view': 'cpsskins_edit_form',
     'filter_content_types': 0,
     'actions': (
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'cpsskins_edit_form',
                  'permissions': ('Manage Themes',)},
                 {'id': 'delete',
                  'name': 'Delete',
                  'action': 'cpsskins_object_delete',
                  'visible': 0,
                  'permissions': ('Manage Themes',)},
                 {'id': 'isstyle',
                  'name': 'isstyle',
                  'action': 'isstyle',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class BaseStyle(PortalContent, PropertyManager):
    """
    Base class for styles
    """

    meta_type = None
    portal_type = None

    isstyle = 1

    manage_options = ( PropertyManager.manage_options      # Properties
                    + (
                       {'label': 'Preview',
                        'action': 'manage_stylePreview'}, 
                      )
                     )

    security = ClassSecurityInfo()

    security.declareProtected(View, 'manage_stylePreview')
    manage_stylePreview = DTMLFile('zmi/manage_stylePreview', globals())

    _actions = factory_type_information[0]['actions']

    _properties = (
        {'id': 'title', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Title'},
        )


    def __init__(self, id, 
                       title= 'Style', 
                       **kw):
        self.id = id
        self.title = title

    security.declareProtected(View,'isPortalStyle')
    def isPortalStyle(self):
        """ is style """
           
        return 1

    security.declareProtected(View,'isPortalContent')
    def isPortalContent(self):
        """ is portal content """
                  
        return 0  

    security.declarePublic('getIconRelative')
    def getIconRelative(self):
        """
        Gets the Icon path, relative to the portal.
        This is needed to catalog correctly in the presence of VHM.
        """
        return self.getIcon(1)

    security.declareProtected('Manage Themes', 'rebuild')
    def rebuild(self):
        """                  
        Rebuild this style
        """              

        for pm in self.propertyMap():
            for p in self._properties:
                 if p['id'] == pm['id']:
                     for k in pm.keys():
                         p[k] = pm.get(k, None)
                             
        prop_dict = {}       
        ids =  self.propertyIds()
        for id in ids:       
           try:              
               prop = self.getProperty(id)
           except:           
               prop = None   

           if prop is None:
               type = self.getPropertyType(id)
               if type in [ 'lines', 'string', 'text']:
                    prop = ' '

               if id == 'visibility':
                    prop = 'always'

           prop_dict[id] = prop

        self.manage_delProperties(ids, REQUEST=None)
        self.manage_changeProperties(prop_dict)

        return                          


    security.declarePrivate('callAction')
    def callAction(self, actionid, **kw):
        """
        Call the given action.
        """
        ti = self.getTypeInfo() 
        try: 
             action = ti.getActionById(actionid)
        except:
             return 
        else:
          meth = self.restrictedTraverse(action)

          # XXX Check permissions

          if getattr(aq_base(meth), 'isDocTemp', 0):
                return apply(meth, (self, self.REQUEST), kw)
          else:
                return apply(meth, (), kw)
          raise 'Not Found', (
            'Cannot find %s action for "%s"' %
            (actionid, self.absolute_url(relative=1)))



    security.declareProtected(View, 'edit_form')
    def edit_form(self, **kw):
        """
        Call the edit action.
        """
        return self.callAction('edit', **kw)

    security.declareProtected('Manage Themes', 'edit')
    def edit(self, **kw):
        """
        Default edit method, changes the properties.
        """
        self.manage_changeProperties(**kw)


    security.declareProtected(View, 'can_delete')
    def can_delete(self):
        """
        Can the style be deleted ?
        """

        return 1


    security.declarePublic('getInfo')
    def getInfo(self):

        infoblock = {}
        actions_list = ['delete', ]

        ti = self.getTypeInfo()
        for actionid in actions_list:  
             actioninfo = {}
             if actionid == 'delete':
                actioninfo['can_delete'] = self.can_delete()
             try:
               action = ti.getActionById(actionid)
               actioninfo['url']  = self.absolute_url() + '/' + self.restrictedTraverse(action).getId()
             except:
               continue 

             infoblock[actionid] = actioninfo
                            
        return infoblock 


    security.declareProtected(View, 'render')
    def render(self, *args, **kw):
        """
        Renders the style in CSS
        """

        return self.callAction('render', **kw)



    security.declareProtected(View, 'preview')
    def preview(self, *args, **kw):
        """
        Renders a preview of the style 
        """

        return self.callAction('preview', **kw)


InitializeClass(BaseStyle)


