# Copyright (c) 2003 Chalmers University of Technology
# Authors: Jean-Marc Orliaguet <jmo@ita.chalmers.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

"""
  BaseTemplet
"""
import string
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from Acquisition import  aq_base, aq_parent
from DateTime import DateTime

from Products.CMFCore.utils import _verifyActionPermissions
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName

def addBaseTemplet(dispatcher, id, REQUEST=None):
    """Add a Base Templet."""
    ob = BaseTemplet(id)
    container = dispatcher.Destination()
    container._setObject(id, ob)
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)


factory_type_information = (
    {'id': 'Base Templet',
     'description': ('A Base Templet is the most basic templet.'),
     'meta_type': 'Base Templet',
     'icon': 'templet_icon.gif',
     'product': 'CPSSkins',
     'factory': 'addBaseTemplet',
     'immediate_view': 'cpsskins_edit_form',
     'filter_content_types': 0,
     'actions': (
                 {'id': 'view',
                  'name': 'View',
                  'action': 'cpsskins_templet_view',
                  'category': 'object',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'cpsskins_edit_form',
                  'visible': 1,
                  'category': 'object',
                  'permissions': ('Manage Themes',)},
                 {'id': 'moveleft',
                  'name': 'Move Left',
                  'action': 'cpsskins_templet_moveleft',
                  'category': 'object',
                  'visible': 0,
                  'permissions': ('Manage Themes',)},
                 {'id': 'moveright',
                  'name': 'Move Right',
                  'action': 'cpsskins_templet_moveright',
                  'visible': 0,
                  'category': 'object',
                  'permissions': ('Manage Themes',)},
                 {'id': 'moveup',
                  'name': 'Move Up',
                  'action': 'cpsskins_templet_moveup',
                  'visible': 0,
                  'category': 'object',
                  'permissions': ('Manage Themes',)},
                 {'id': 'movedown',
                  'name': 'Move Down',
                  'action': 'cpsskins_templet_movedown',
                  'visible': 0,
                  'category': 'object',
                  'permissions': ('Manage Themes',)},
                 {'id': 'toggle',
                  'name': 'Toggle',
                  'action': 'cpsskins_templet_toggle',
                  'visible': 0,
                  'category': 'object',
                  'permissions': ('Manage Themes',)},
                 {'id': 'delete',
                  'name': 'Delete',
                  'action': 'cpsskins_object_delete',
                  'visible': 0,
                  'category': 'object',
                  'permissions': ('Manage Themes',)},
                 {'id': 'isportaltemplet',
                  'name': 'isportaltemplet',
                  'action': 'isportaltemplet',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class BaseTemplet(PortalContent, PropertyManager):
    """
    Base class for templets.
    """

    meta_type = None
    portal_type = None

    isportaltemplet = 1

    manage_options = ( PropertyManager.manage_options     # Properties
                     + PortalContent.manage_options[2:3]  # View
                     )

    security = ClassSecurityInfo()

    _actions = factory_type_information[0]['actions']

    _properties = (
        {'id': 'title', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Title', 
         'category': 'general'},
        {'id': 'closed', 
         'type': 'boolean', 
         'mode': 'w', 
         'label': 'Closed', 
         'category': 'none'},
        {'id': 'cacheable', 
         'type': 'boolean', 
         'mode': 'w', 
         'label': 'Cacheable', 
         'category': 'general'},
        {'id': 'hidden_in_text_mode', 
         'type': 'boolean', 
         'mode': 'w', 
         'label': 'Hidden in text mode', 
         'category': 'general'},
        {'id': 'display_title_in_text_mode', 
         'type': 'boolean', 
         'mode': 'w', 
         'label': 'Display title in text mode', 
         'category': 'general'},
        {'id': 'xpos', 
         'type': 'int', 
         'mode': 'w', 
         'label': 'xpos',  
         'category': 'none'},
        {'id': 'align', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Horizontal alignment', 
         'select_variable': 'cpsskins_alignList', 
         'category': 'layout'},
        {'id': 'padding', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Templet padding', 
         'category': 'layout'},
        {'id': 'margin', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Templet margin',  
         'category': 'layout'},
        {'id': 'templet_height', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Templet height', 
         'category': 'layout'},
        {'id': 'fontcolor', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Font color', 
         'select_variable': 'cpsskins_listFontColors', 
         'meta_type': 'Font Color', 
         'category': 'style'},
        {'id': 'fontshape', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Font shape', 
         'select_variable': 'cpsskins_listFontShapes', 
         'meta_type': 'Font Shape', 
         'category': 'style'},
        {'id': 'shape', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Templet shape', 
         'select_variable': 'cpsskins_listAreaShapes', 
         'meta_type': 'Area Shape', 
         'category': 'style'},
        {'id': 'color', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Templet color', 
         'select_variable': 'cpsskins_listAreaColors', 
         'meta_type': 'Area Color',  
         'category': 'style'},
        {'id': 'visibility',  
         'type': 'selection',  
         'mode': 'w', 
         'label': 'Visibility criteria', 
         'select_variable': 'cpsskins_visibilityList', 
         'category': 'visibility'},
        {'id': 'visibility_paths',  
         'type': 'multiple selection',   
         'mode': 'w', 
         'label': 'The visibility paths', 
         'select_variable': 'cpsskins_listPaths', 
         'category': 'visibility'},
        {'id': 'languages', 
         'type': 'multiple selection', 
         'mode': 'w', 
         'label': 'The languages in which it is visible', 
         'select_variable': 'cpsskins_listLanguages', 
         'category': 'visibility'},
        )


    def __init__(self, id, 
                       title = 'Templet',
                       closed = 0,
                       cacheable = 1,
                       hidden_in_text_mode = 0,
                       display_title_in_text_mode = 0,
                       visibility = 'always',
                       visibility_paths= [],
                       languages = [],
                       xpos = 0, 
                       align = 'left', 
                       templet_height = '',
                       padding = '0.5em', 
                       margin = '0.5em', 
                       fontshape = 'Arial',
                       fontcolor = 'Black',
                       shape = 'NoBorder',
                       color = 'Transparent',
                       **kw):
        self.id = id
        self.closed = closed
        self.cacheable = cacheable
        self.hidden_in_text_mode = hidden_in_text_mode
        self.display_title_in_text_mode = display_title_in_text_mode
        self.visibility = visibility
        self.visibility_paths = visibility_paths
        self.languages = languages
        self.title = title
        self.xpos = int(xpos)
        self.align = align
        self.templet_height = templet_height
        self.padding = padding
        self.margin = margin
        self.fontshape = fontshape
        self.fontcolor = fontcolor
        self.shape = shape
        self.color = color

    security.declareProtected(View,'isPortalTemplet')
    def isPortalTemplet(self):
        """ is portal templet """
           
        return 1

    security.declareProtected(View,'isPortalContent')
    def isPortalContent(self):
        """ is portal content """
           
        return 0

    security.declareProtected('Manage Themes', 'getTempletPosition')
    def getTempletPosition(self):
        """ 
        Return the page block's ypos in the theme folder
        """

        container = self.aq_parent
        return container.get_object_position(self.getId())



    security.declarePublic('getIconRelative')
    def getIconRelative(self):
        """
        Gets the Icon path, relative to the portal.
        This is needed to catalog correctly in the presence of VHM.
        """
        return self.getIcon(1)

    security.declareProtected('Manage Themes', 'rebuild')
    def rebuild(self):
        """                  
        Rebuild this templet
        """              

        for pm in self.propertyMap():
            for p in self._properties:
                 if p['id'] == pm['id']:
                     for k in pm.keys():
                         p[k] = pm.get(k, None)
                             
        prop_dict = {}       
        ids =  self.propertyIds()
        for id in ids:       
           try:              
               prop = self.getProperty(id)
           except:           
               prop = None   

           if prop is None:
               type = self.getPropertyType(id)
               if type in ['int']:
                    prop = 0

               if type in ['lines', 'string', 'text', 'selection', 'multiple selection' ]:
                    prop = ''

               if id == 'visibility':
                    prop = 'always'

               if id == 'render_html':
                    prop = 1


           prop_dict[id] = prop
        self.manage_delProperties(ids, REQUEST=None)
        try:
           self.manage_changeProperties(prop_dict)
        except:
           pass 

        return 


    security.declarePublic('getVisibility')
    def getVisibility(self, REQUEST=None):

        if self.closed: 
            return 

        if self.languages:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            if hasattr(portal, 'Localizer'):
                lctool = portal.Localizer
            else:
                lctool = getToolByName(self, 'portal_messages')
            if lctool is not None:
                selected_language = lctool.get_selected_language()
                lang_list = self.languages

                if selected_language not in lang_list:
                   return

        visibility = self.visibility
        if visibility == 'always':
            return 1

        if visibility == 'time_limited':
            now = DateTime()
            effective_date = self.EffectiveDate()
            expiration_date = self.ExpirationDate()

            if effective_date != 'None':
               if now < DateTime(effective_date):
                  return 0

            if expiration_date != 'None':
               if now >= DateTime(expiration_date):
                  return 0
            return 1

        if visibility == 'if_authenticated':
            mtool = getToolByName(self, 'portal_membership')
            if not mtool.isAnonymousUser():
                return 1

        if visibility == 'if_anonymous':
            mtool = getToolByName(self, 'portal_membership')
            if mtool.isAnonymousUser():
                return 1

        if visibility in ['only_in', 'everywhere_except_in', 'starting_from', 'up_till']:
            utool = getToolByName(self, 'portal_url')
            portal_url = utool(relative=0)

            if REQUEST is None:
                return

            # simulated URL
            rurl = REQUEST.form.get('sim_url', None)
            if rurl:
               url =  portal_url + rurl

            # real URL
            if rurl is None:
               url_obj = REQUEST.get('context_obj', None)
               if url_obj:
                    url = utool.getRelativeContentURL(url_obj)  
                    if url: 
                        url = '/' + url + '/'
                    else:
                        url = '/'
            
            if url is None:
               return

            url_path = url

            paths = [(p == '/' and '' or p) for p in self.visibility_paths]

            if visibility == 'only_in':
                if url_path in paths:
                      return 1

            if visibility == 'everywhere_except_in':
                if url_path not in paths:
                      return 1

            if visibility == 'starting_from':
               for p in paths:
                   if url_path.startswith(p):
                      return 1

            if visibility == 'up_till':
               for p in paths:
                   if url_path.startswith(p) and url_path != p:
                      return 0
               return 1


 
        if visibility == 'if_secure_connection':
            if REQUEST is not None:
                theBase = REQUEST['BASE0'].split('//')[0]
                if theBase == 'https:': 
                    return 1

          

    security.declarePublic('getInfo')
    def getInfo(self):

        infoblock = {}
        actions_list = ['moveup', 'movedown', 'moveleft', 'moveright', 'toggle', 'delete']

        ti = self.getTypeInfo()
        for actionid in actions_list:  
             actioninfo = {}
             if actionid == 'moveup':
                 actioninfo['can_move'] = self.can_moveup()
             if actionid == 'moveleft':
                 actioninfo['can_move'] = self.can_moveleft()
             if actionid == 'moveright':
                 actioninfo['can_move'] = self.can_moveright()
             if actionid == 'movedown':
                 actioninfo['can_move'] = self.can_movedown()
             if actionid == 'toggle':
                 actioninfo['can_toggle'] = self.can_toggle()
             if actionid == 'delete':
                 actioninfo['can_delete'] = self.can_delete()
             try:
                action = ti.getActionById(actionid)
                actioninfo['url']  = self.absolute_url() + '/' + self.restrictedTraverse(action).getId()
             except:
                continue
             infoblock[actionid] = actioninfo
                            
        return infoblock 


    security.declarePrivate('callAction')
    def callAction(self, actionid, **kw):
        """
        Call the given action.
        """
        ti = self.getTypeInfo() 
        try: 
             action = ti.getActionById(actionid)
        except:
             return
        else:
            try:
                meth = self.restrictedTraverse(action)
            except:
                return 
            else:

                # XXX Check permissions
                if getattr(aq_base(meth), 'isDocTemp', 0):
                    return apply(meth, (self, self.REQUEST), kw)
                else:
                    return apply(meth, (), kw)
                return apply(meth, (), kw)
                raise 'Not Found', (
                'Cannot find %s action for "%s"' %
                (actionid, self.absolute_url(relative=1)))



    security.declareProtected(View, 'edit_form')
    def edit_form(self, **kw):
        """
        Call the edit action.
        """
        return self.callAction('edit', **kw)

    security.declareProtected('Manage Themes', 'edit')
    def edit(self, **kw):
        """
        Default edit method, changes the properties.
        """
        self.manage_changeProperties(**kw)


    security.declareProtected(View, 'moveleft_pos')
    def moveleft_pos(self):
        """
        Can the templet be moved to the left?
        """

        if self.can_moveleft():
           return self.xpos -1 
        return 


    security.declareProtected(View, 'moveright_pos')
    def moveright_pos(self):
        """
        Return the new position 
        """

        if self.can_moveright():
            return self.xpos + 1
        return 

    security.declareProtected(View, 'moveup_pos')
    def moveup_pos(self):
        """
        Return the new position 
        """

        container = self.aq_parent
        this_pos = self.get_object_position(self.getId())
        newpos = -1
        for obj in container.objectValues():
            meth = getattr(obj, 'isPortalTemplet', None)
            if callable(meth) and meth():
               if obj.xpos == self.xpos:
                  pos = self.get_object_position(obj.getId())
                  if pos > newpos and pos < this_pos:
                     newpos = pos

        return newpos

    security.declareProtected(View, 'movedown_pos')
    def movedown_pos(self):
        """
        Return the new position 
        """

        container = self.aq_parent
        this_pos = self.get_object_position(self.getId())
        newpos = -1
        for obj in container.objectValues():
            meth = getattr(obj, 'isPortalTemplet', None)
            if callable(meth) and meth():
               if obj.xpos == self.xpos:
                  pos = self.get_object_position(obj.getId())
                  if  pos > this_pos:
                     newpos = pos
                     break

        return newpos


    security.declareProtected(View, 'can_toggle')
    def can_toggle(self):
        """
        Can the templet be toggled ( open/close ) ?
        """
      
        if hasattr(self, 'closed'):
            return 1


    security.declareProtected(View, 'can_delete')
    def can_delete(self):
        """
        Can the templet be deleted ?
        """

        return 1


    security.declareProtected(View, 'can_moveup')
    def can_moveup(self):
        """
        Can the templet be moved up?
        """

        # find a page block above it
        container = self.aq_parent
        if container.can_moveup():
            return 1

        # otherwise find a templet above it
        this_pos = container.get_object_position(self.getId())
        for obj in container.objectValues():
            meth = getattr(obj, 'isPortalTemplet', None)
            if callable(meth) and meth():
               if obj.xpos == self.xpos:
                  pos = container.get_object_position(obj.getId())
                  if pos < this_pos:
                     return 1
	return

    security.declareProtected(View, 'can_movedown')
    def can_movedown(self):
        """
        Can the templet be moved down?
        """

        # find a page block below it
        container = self.aq_parent
        if container.can_movedown():
            return 1

        container = self.aq_parent
        this_pos = self.get_object_position(self.getId())
        for obj in container.objectValues():
            meth = getattr(obj, 'isPortalTemplet', None)
            if callable(meth) and meth():
               if obj.xpos == self.xpos:
                  pos = self.get_object_position(obj.getId())
                  if pos > this_pos:
                     return 1
	return


    security.declareProtected(View, 'can_moveright')
    def can_moveright(self):
        """
        Can the templet be moved to the right?
        """

        container = self.aq_parent 
        maxcols = getattr(container, 'maxcols', None)
        if maxcols is None:
            return 
        if self.xpos < maxcols -1 :
            return 1
        return 


    security.declareProtected(View, 'can_moveleft')
    def can_moveleft(self):
        """
        Can the templet be moved to the left?
        """

        if self.xpos > 0:
            return 1
        return 

    security.declareProtected('Manage Themes', 'toggle')
    def toggle(self, state):
        """
        Open / close the templet
        """

        self.closed = state

        return 


    security.declareProtected('Manage Themes', 'duplicate')
    def duplicate(self):
        """
        Duplicate a Templet
        """
        
        container = self.aq_parent
        src_ypos = container.get_object_position(self.getId())
        newid = str(int(DateTime()))
        container.manage_clone(self, newid)
        newobj  = getattr(container, newid, None)
        container.move_object_to_position(newobj.getId(), src_ypos + 1)

        # needed for the ImageBox (bug in CMF?) 
        setattr(newobj, 'id', newid)

        return newobj



    security.declareProtected('Manage Themes', 'set_property')
    def set_property(self, prop=None, value=None):
        """
        Sets a property 
        """

        if value is None:
           return
     
        if not hasattr(self, prop):
           return
        setattr(self, prop, value)


    security.declareProtected('Manage Themes', 'move_to_pageblock')
    def move_to_pageblock(self, pageblock=None, xpos=None, ypos=None):
        """
        Move the templet to a given pageblock 
        returns the instance of the moved object
        """

        if pageblock is None:
            return
        if xpos is None:
            return
        if ypos is None:
            return

        id = self.getId()

        current_xpos = self.xpos
        new_xpos = int(xpos)
        self.xpos = new_xpos
        container = self.aq_parent
        src_pageblock = container.getId()
        theme_container = container.aq_parent
        current_ypos = container.get_object_position(self.getId())

        for obj in theme_container.objectValues():
           if obj.getId() == pageblock:
              cookie = container.manage_cutObjects(self.getId())
              obj.manage_pasteObjects(cookie) 

              for templet in obj.objectValues():
                 if templet.getId() == id:
                    newpos = int(ypos) 
                    if pageblock == src_pageblock:
                       if newpos > current_ypos and new_xpos != current_xpos:
                           newpos = newpos -1;
                    obj.move_object_to_position(id, newpos)

                    return templet


    security.declareProtected('Manage Themes', 'move')
    def move(self, direction=None):
        """
        Move the templet into a direction 
        returns the instance of the moved object
        """
  
        container = self.aq_parent 
        if direction == 'left' and self.can_moveleft():
            self.xpos = self.moveleft_pos()
            return self
 
        if direction == 'right' and self.can_moveright():
            self.xpos = self.moveright_pos()
            return self

        if direction == 'up' and self.can_moveup():
            newpos = self.moveup_pos()
            if newpos >= 0:
                container.move_object_to_position(self.getId(), newpos)
                return self
            else:
                cookie = container.manage_cutObjects(self.getId())
                pos =  container.moveup_pos()
                theme_container = container.aq_parent
                for obj in theme_container.objectValues():
                    if theme_container.get_object_position(obj.getId()) == pos:
                        obj.manage_pasteObjects(cookie)
                        maxcols = getattr(obj, 'maxcols', None)
                        if maxcols is not None and self.xpos > maxcols-1:
                             self.xpos = maxcols - 1
                        return getattr(obj, self.getId(), None) 

        if direction == 'down' and self.can_movedown():
            newpos = self.movedown_pos()
            if newpos >= 0:
                container.move_object_to_position(self.getId(), newpos)
                return self
            else:
                cookie = container.manage_cutObjects(self.getId())
                pos =  container.movedown_pos()
                theme_container = container.aq_parent
                for obj in theme_container.objectValues():
                    if theme_container.get_object_position(obj.getId()) == pos:
                        obj.manage_pasteObjects(cookie)
                        newobj =  getattr(obj, self.getId(), None) 
                        obj.move_object_to_position(newobj.getId(), int(0))

                        maxcols = getattr(obj, 'maxcols', None)
                        if maxcols is not None and self.xpos > maxcols-1:
                             self.xpos = maxcols - 1

                        return newobj
        return 


    security.declareProtected(View, 'render')
    def render(self, *args, **kw):
        """
        Renders the templet.
        """

        rendered = self.callAction('render', **kw)
        if rendered is None:
           rendered = self.cpsskins_brokentemplet()
 
        return rendered



InitializeClass(BaseTemplet)


