# Copyright (c) 2003 Chalmers University of Technology
# Authors: Jean-Marc Orliaguet <jmo@ita.chalmers.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

"""
  Portal Theme
"""

from Globals import InitializeClass, DTMLFile
from AccessControl import getSecurityManager, ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from OFS.Folder import Folder
from Acquisition import aq_base, aq_parent, aq_inner

from Products.CMFCore.utils import _verifyActionPermissions
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent, ManagePortal

from Products.PortalContentFolder.PortalContentFolder import PortalContentFolder


def addPortalTheme(dispatcher, id, REQUEST=None):
    """Add a Portal Theme."""
    ob = PortalTheme(id)
    container = dispatcher.Destination()
    container._setObject(id, ob)
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)


factory_type_information = (
    {'id': 'Portal Theme',
     'title': 'Portal Theme',
     'description': ('A Portal Theme contains Page Blocks.'),
     'meta_type': 'Portal Theme',
     'icon': 'portaltheme.gif',
     'product': 'CPSSkins',
     'factory': 'addPortalTheme',
     'immediate_view': 'cpsskins_edit_form',
     'filter_content_types': 0,
     'global_allow': 0,
     'actions': (
                 {'id': 'view',
                  'name': 'View',
                  'action': 'cpsskins_default_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_Edit_',
                  'action': 'cpsskins_edit_form',
                  'visible': 0,
                  'permissions': ('Manage Themes',)
                 },
                 {'id': 'editstyles',
                  'name': '_Edit styles_',
                  'action': 'cpsskins_edit_styles',
                  'visible': 0,
                  'permissions': ('Manage Themes',)
                 },
                 {'id': 'addpageblock',
                  'name': '_action_addpageblock_',
                  'action': 'cpsskins_addpageblock_form',
                  'visible': 0,
                  'permissions': ('Manage Themes',),
                  'category': 'object'},
                 ),
     },
    )


class PortalTheme(PortalContentFolder):
    """
    Class for Portal Themes.
    """

    meta_type = "Portal Theme"
    portal_type = "Portal Theme"

    manage_options = ( 
                      Folder.manage_options[0:1] +
                      Folder.manage_options[2:3] +
                      (
                       {'label': 'Rebuild',
                        'action': 'manage_themeRebuild'},
                       ) 
                     )

    security = ClassSecurityInfo()

    security.declareProtected('Manage Themes', 'manage_themeRebuild')
    manage_themeRebuild = DTMLFile('zmi/manage_themeRebuild', globals())


    _properties = (
        {'id': 'title', 'type':'string', 'mode':'w', 'label':'Title', 'category' : 'general'},
        {'id': 'margin', 'type': 'string', 'mode': 'w', 'label': 'Margin', 'category' : 'general'},
        {'id': 'default', 'type': 'boolean', 'mode': 'w', 'label': 'Default', 'category' : 'none'},
        {'id': 'align', 'type': 'selection', 'mode': 'w', 'label': 'Align', 'select_variable': 'cpsskins_alignList', 'category' : 'general'},
        {'id': 'color', 'type': 'selection', 'mode': 'w', 'label': 'Color', 'select_variable': 'cpsskins_listAreaColors', 'category' : 'general'},
        {'id': 'shortcut_icon', 'type': 'selection', 'mode': 'w', 'label': 'Shortcut icon', 'select_variable' : 'cpsskins_listIcons',  'category': 'general', 'image' : 'icons'},
        {'id': 'theme_renderer', 'type': 'selection', 'mode': 'w', 'label': 'Theme renderer', 'select_variable' : 'cpsskins_listThemeRenderers',  'category': 'general' },
        {'id': 'author', 'type':'text', 'mode':'w', 'label':'Author', 'category' : 'general'},
        {'id': 'copyright', 'type':'text', 'mode':'w', 'label':'Copyright', 'category' : 'general'},
        {'id': 'license', 'type':'string', 'mode':'w', 'label':'License', 'category' : 'general'},
        )

    def __init__(self, id, 
                       title = 'Portal Theme',
                       color = 'Transparent',
                       align = 'center',
                       margin = '0.5em',
                       default = 0,
                       shortcut_icon = '',
                       theme_renderer = 'default',
                       author = '',
                       copyright = '',
                       license = '',
                       **kw):
        self.id = id
        self.title = title
        self.color = color
        self.align = align
        self.margin = margin
        self.default = default
        self.shortcut_icon = shortcut_icon
        self.theme_renderer = theme_renderer
        self.author = author
        self.copyright = copyright
        self.license = license


    security.declareProtected(View,'isPortalTheme')
    def isPortalTheme(self):
        """ is a portal theme """
           
        return 1

    security.declareProtected(View,'isPortalContent')
    def isPortalContent(self):
        """ is portal content """
                  
        return 0  

    security.declareProtected('Manage Themes', 'manage_rebuild')
    def manage_rebuild(self, REQUEST=None):
        """
        Rebuild this theme
        """

        self.rebuild()
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')


    security.declareProtected('Manage Themes', 'rebuild')
    def rebuild(self):
        """
        Rebuild this theme's properties
        """

        themefolders = ( 'styles', 'icons', 'backgrounds', 'palettes' )

        for themefolder in themefolders:
            backupid = themefolder + '-bak'
            if hasattr(self, backupid):
                self.manage_delObjects([backupid])

            if not hasattr(self, themefolder):
                self.invokeFactory('Theme Folder', id=themefolder)
                continue

            if hasattr(self, themefolder):
                folder = getattr(self, themefolder)
                meth = getattr(folder, 'isThemeFolder', None)
                if callable(meth) and meth():
                    continue

                self.manage_renameObjects([themefolder], [backupid])
                backup_folder = getattr(self, backupid)
                objects = backup_folder.objectIds()
                cookie = backup_folder.manage_cutObjects(objects)
                self.invokeFactory('Theme Folder', themefolder)
                new_folder = getattr(self, themefolder)
                new_folder.manage_pasteObjects(cookie)
                self.manage_delObjects([backupid])


        for pm in self.propertyMap():
            for p in self._properties:
                 if p['id'] == pm['id']:
                     for k in pm.keys():
                         p[k] = pm.get(k, None)

        prop_dict = {}
        ids =  self.propertyIds()
        for id in ids:    
           try:
               prop = self.getProperty(id)
           except:
               prop = None
           prop_dict[id] = prop

        self.manage_delProperties(ids, REQUEST=None)
        self.manage_changeProperties(prop_dict)

        for obj in self.objectValues():
           meth = getattr(obj, 'isPortalPageBlock', None)
           if callable(meth) and meth():
                  obj.rebuild()

        styles_dir = getattr(self, 'styles', None)
        
        if styles_dir is None:
             return
        for obj in styles_dir.objectValues():
           meth = getattr(obj, 'isStyle', None)
           if callable(meth) and meth():
                  obj.rebuild()



    security.declarePrivate('callAction')
    def callAction(self, actionid, **kw):
        """
        Call the given action.
        """

        ti = self.getTypeInfo() 
        try: 
             action = ti.getActionById(actionid)
        except:
             return 
        else:
          meth = self.restrictedTraverse(action)

          # XXX Check permissions

          if getattr(aq_base(meth), 'isDocTemp', 0):
                return apply(meth, (self, self.REQUEST), kw)
          else:
                return apply(meth, (), kw)
          raise 'Not Found', (
            'Cannot find %s action for "%s"' %
            (actionid, self.absolute_url(relative=1)))


    security.declarePublic('getPageBlocks')
    def getPageBlocks(self, theme=None):
        """ returns a list of page blocks sorted by ypos"""

        pageblocks = []                
	for v in self.objectValues():
            meth = getattr(v, 'isPortalPageBlock', None)
            if callable(meth) and meth():
                  pageblocks.append(v)   

        return pageblocks
	
    security.declareProtected('Manage Themes', 'addpageblock_form')
    def add_pageblock_form(self, **kw):
        """
        Call the add templet action.
        """
        return self.callAction('addtemplet', **kw)


    security.declareProtected('Manage Themes', 'edit_form')
    def edit_form(self, **kw):
        """
        Call the edit action.
        """
        return self.callAction('edit', **kw)


    security.declareProtected('Manage Themes', 'edit')
    def edit(self, **kw):
        """
        Default edit method, changes the properties.
        """
        self.manage_changeProperties(**kw)


InitializeClass(PortalTheme)


