# Copyright (c) 2003 Chalmers University of Technology
# Authors: Jean-Marc Orliaguet <jmo@ita.chalmers.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

"""
  PortalThemesTool
"""

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo, Unauthorized
from Acquisition import aq_parent

from OFS.Folder import Folder
from Products.CMFCore.PortalFolder import PortalFolder

from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore import DirectoryView
from Products.CMFCore.Expression import Expression
from Products.CMFCore.utils import getToolByName

import QuickImporter

CPSSKINS_THEME_COOKIE_NAME = 'cpsskins_theme'
CPSSKINS_RAMCACHE_ID = 'cpsskins-templetcache'
CPSSKINS_RENDER_METHOD_ID = 'cpsskins_render'

class PortalThemesTool(PortalFolder, ActionProviderBase):
    """
    Portal Themes Tool
    """

    id = 'portal_themes'
    meta_type = 'Portal Themes Tool'


    _actions = [ActionInformation(id='configThemes'
                            , title='_action_themes_reconfig_'
                            , description='Configure Portal Themes'
                            , action=Expression(
            text='string: ${portal_url}/cpsskins_themes_reconfig_form')
                            , permissions=('Manage Themes',)
                            , category='global'
                            , condition=None
                            , visible=1
                             ),
                             ] 


    security = ClassSecurityInfo()

    manage_options = ( Folder.manage_options[0:1]
                     + ( {'label': 'Default theme', 'action': 'manage_selectDefaultTheme'}, )
                     + ( {'label': 'Rebuild', 'action': 'manage_themesRebuild'}, )
                     + ( {'label' : 'Overview', 'action' : 'manage_overview' }, )
                     + ActionProviderBase.manage_options 
                     )

    security = ClassSecurityInfo()

    security.declareProtected('Manage Themes', 'manage_themesRebuild')
    manage_themesRebuild = DTMLFile('zmi/manage_themesRebuild', globals())

    security.declareProtected('Manage Themes', 'manage_selectDefaultTheme')
    manage_selectDefaultTheme = DTMLFile('zmi/manage_selectDefaultTheme', globals())


    def __init__(self):
        PortalFolder.__init__(self, self.id)

    #
    #   ActionProvider interface
    #
    security.declarePublic('listActions')
    def listActions(self, info=None):
        """
        Return actions provided via tool.
        """
        return self._actions 

    #
    # ZMI
    #
    security.declareProtected('Manage Themes', 'manage_overview')
    manage_overview = DTMLFile('zmi/explainPortalThemesTool', globals())

    #
    # Public API
    #

    security.declarePublic('getThemeCookieID')
    def getThemeCookieID(self):
        """ Gets the cookie ID used to set themes """
        return CPSSKINS_THEME_COOKIE_NAME


    security.declarePublic('getPortalThemeRoot')
    def getPortalThemeRoot(self, object=None):
        """ Gets the portal theme root container of a given object """

        if object is None:
            return
        rurl = object.absolute_url(relative=1)
        path = rurl.split('/')
        a = []

        for p in range(1,len(path)+1):
            try:
               container =  self.restrictedTraverse(path[:p])
               meth = getattr(container.aq_explicit, 'isPortalTheme', None)
               if callable(meth) and meth():
                   theme_name = container.getId()
                   theme_container = self.getThemeContainer(theme=theme_name)
                   return theme_container
            except:
               return


    security.declarePublic('listStyles')
    def listStyles(self, category=None, object=None ):
        """ Gets the list of available styles """

        style = {}
        title_list = []
        object_list = []
        if object is None:
             return

        themeroot = self.getPortalThemeRoot(object)
        if themeroot is None:
             return

        styles_dir = getattr(themeroot, 'styles', None)
        if styles_dir is None:
            return

        for obj in styles_dir.objectValues():
            if getattr(obj.aq_explicit, 'meta_type', None) == category:
                title_list.append(obj.title)
                object_list.append(obj)

        style['title'] = title_list
        style['object'] = object_list
        return style

    security.declarePublic('listPalettes')
    def listPalettes(self, category=None, object=None ):
        """ Gets the list of available palettes for a given object"""

        palette = {}
        title_list = []
        object_list = []
        if object is None:
             return

        themeroot = self.getPortalThemeRoot(object)
        if themeroot is None:
             return

        palettes_dir = getattr(themeroot, 'palettes', None)
        if palettes_dir is None:
            return

        for obj in palettes_dir.objectValues():
            #meth = getattr(obj.aq_explicit, 'is%s' % category, None)
            #if callable(meth) and meth():
            if getattr(obj.aq_explicit, 'meta_type', None) == category:
                title_list.append(obj.title)
                object_list.append(obj)

        palette['title'] = title_list
        palette['object'] = object_list
        return palette


    security.declarePublic('getPageBlocks')
    def getPageBlocks(self, theme=None):
        """ returns a list of page blocks sorted by ypos"""

        theme_container =  self.getEffectiveThemeContainer(theme=theme)
        if theme_container is None:
            return None

        pageblocks = []                
	for v in theme_container.objectValues():
            meth = getattr(v, 'isPortalPageBlock', None)
            if callable(meth) and meth():
                    pageblocks.append(v)   

        return pageblocks

    
    security.declarePublic('getThemeContainer')
    def getThemeContainer(self, theme='default', parent=None ):
        """Gets the themes container.
	   - theme = 'printable' | 'default' | ... 
	   - theme = None : will be interpreted as theme = 'default'
           - parent = 1 : returns the parent 'themes' folder
        """

        theme_container = getToolByName(self, 'portal_themes')
        if theme_container is None:
            return

        if parent:
            return theme_container

        if theme is not None and theme in theme_container.objectIds():
	    return getattr(theme_container, theme, None)

        return theme_container[0]


    security.declarePublic('getDefaultThemeName')
    def getDefaultThemeName(self, REQUEST=None):
        """ gets the default theme
        """
 
        themes =  self.getGlobalThemes() 
        for theme in themes:
           if getattr(theme, 'default', None):
               if theme.default is not None:
                   return theme.getId()
        if len(themes) > 0:
           return themes[0].getId()


    security.declareProtected('Manage Themes', 'setDefaultTheme')
    def setDefaultTheme(self, default_theme=None, REQUEST=None):
        """ sets the default theme
        """
 
        themes =  self.getGlobalThemes() 
        for theme in themes:
             if theme.getId() == default_theme:
                theme.default = 1
             else:
                theme.default = 0
 

    security.declarePublic('getRAMCache')
    def getRAMCache(self, REQUEST=None):
        """ returns the RAM cache
        """

        portal = self.portal_url.getPortalObject()
        if hasattr(portal, CPSSKINS_RAMCACHE_ID):
           return portal[CPSSKINS_RAMCACHE_ID]




    security.declareProtected('Manage Themes', 'activateRAMCache')
    def activateRAMCache(self, REQUEST=None):
        """ Activate the RAM cache
        """

        portal = self.portal_url.getPortalObject()
        skinstool = getToolByName(self, 'portal_skins')
                
        custom_skin = skinstool.custom
        CPSSkins_skin = skinstool.CPSSkins

        if CPSSKINS_RENDER_METHOD_ID in custom_skin.objectIds():
            custom_skin._delObject(CPSSKINS_RENDER_METHOD_ID)
                
        if CPSSKINS_RAMCACHE_ID in portal.objectIds():
            portal._delObject(CPSSKINS_RAMCACHE_ID)
                
        try:    
            portal.manage_addProduct['StandardCacheManagers'].manage_addRAMCacheManager(CPSSKINS_RAMCACHE_ID)
        except: 
            pass
               
        settings = { 'threshold': '100', 
               'cleanup_interval': '500', 
               'request_vars' : ('AUTHENTICATED_USER', ), 
               'max_age':'1000'} 
                
        templetcache = portal[CPSSKINS_RAMCACHE_ID]
        templetcache.manage_editProps(title='Templet RAM cache', \
                settings = settings)
             
        CPSSkins_skin[CPSSKINS_RENDER_METHOD_ID].manage_doCustomize(folder_path='custom')    
        render_method = getattr(custom_skin, CPSSKINS_RENDER_METHOD_ID, None)
        render_method.ZCacheable_setManagerId(manager_id = CPSSKINS_RAMCACHE_ID)



    security.declareProtected('Manage Themes', 'deactivateRAMCache')
    def deactivateRAMCache(self, REQUEST=None):
        """ Deactivate the RAM cache
        """

        portal = self.portal_url.getPortalObject()
        skinstool = getToolByName(self, 'portal_skins')
                
        custom_skin = skinstool.custom
        CPSSkins_skin = skinstool.CPSSkins

        if CPSSKINS_RENDER_METHOD_ID in custom_skin.objectIds():
            custom_skin._delObject(CPSSKINS_RENDER_METHOD_ID)
                
        if CPSSKINS_RAMCACHE_ID in portal.objectIds():
            portal._delObject(CPSSKINS_RAMCACHE_ID)



    security.declarePublic('getRequestedThemeName')
    def getRequestedThemeName(self, REQUEST=None):
        """Gets the name of the requested theme by checking if there is a 'cpsskins_theme' cookie, ?pp=1, or ?theme=...
	   in this order : cookie > theme > pp
	"""

        if REQUEST is None:
	    return None
	FORM = REQUEST.form
	COOKIES = REQUEST.cookies

        theme = self.getDefaultThemeName()

        if hasattr(REQUEST, CPSSKINS_THEME_COOKIE_NAME ):
	   theme = COOKIES[CPSSKINS_THEME_COOKIE_NAME]

        if hasattr(REQUEST, 'theme' ):
	   theme = FORM['theme']

        if hasattr(REQUEST, 'pp' ):
	   theme = 'printable'

        return theme


    def getEffectiveThemeContainer(self, theme=None):
        """Gets a theme container depending on who is requesting it and what is available.
	"""

        if theme is not None:
	    theme_container = self.getThemeContainer(theme=theme)

            # theme explicitly specified
	    if theme_container is not None:
	          return theme_container

	default_theme_container = self.getThemeContainer(theme='default')
        return default_theme_container
        


    security.declarePublic('getGlobalThemes')
    def getGlobalThemes(self):
        """Gets the list of global themes ('default', 'printable', ... )
	   returns a list of names.
	"""
     
        themes_container = self.getThemeContainer(parent=1)
        atool = getToolByName(self, 'portal_actions')
        theme_list = []

	if themes_container:
           for theme in themes_container.objectValues():
               meth = getattr(theme.aq_explicit, 'isPortalTheme', None)
               if callable(meth) and meth():
                      theme_list.append(theme)

        return theme_list

    security.declareProtected(View, 'getIconFor')
    def getIconFor(self, category, id):
        """ Returns an action icon - based on CMFActionsIcons
        """
        icons = {}
        actionicons = getToolByName(self, 'portal_actionicons', None)

        if actionicons:
            try:
               iconinfo = actionicons.getActionIcon(category, id)
               icon = icons.setdefault( (category, id), iconinfo )
            except:
               pass
            else:
               return icon


    security.declareProtected('Manage Themes', 'delObject')
    def delObject(self, object=None):
 
        if object is None:
           return

        container = object.aq_parent

        mtool = getToolByName(self, 'portal_membership')
        if not mtool.checkPermission('Manage Themes', container):
            raise Unauthorized
        container.manage_delObjects(object.getId())

        return 


    security.declareProtected('Manage Themes', 'importTheme')
    def importTheme(self, file=None, REQUEST=None):
        """ Imports a theme from a .zexp file
            Returns the its id
        """

        if file is None:
            return
 
        themes_container = self.getThemeContainer(parent=1)

        tmp_id = 'tmp'
        new_tmp_id = tmp_id
        tmp_dir = getattr(themes_container, tmp_id, None)

        if tmp_dir is None:
           themes_container.manage_addFolder(tmp_id)
        else:
           ids = themes_container.objectIds()
           i = 0
           while 1:
              if ids is None: 
                 break
              if new_tmp_id not in ids:
                 break 
              i = i + 1
              new_tmp_id = tmp_id + str(i)

           themes_container.manage_renameObjects([tmp_id], [new_tmp_id])
 
        tmp_dir = getattr(themes_container, new_tmp_id, None)
        if tmp_dir is None:
           return 

        try:
            QuickImporter.manage_doQuickImport(tmp_dir, file, set_owner=0, leave=0, REQUEST=None)
        except:
            return
        theme_id = tmp_dir.objectIds()[0]

        ids = themes_container.objectIds()
        new_id = theme_id
        i = 0
        while 1:
          if ids is None: 
              break
          if new_id not in ids:
              break 
          i = i + 1
          new_id = theme_id + str(i)

        if new_id != theme_id: 
           tmp_dir.manage_renameObjects([theme_id], [new_id])

        cookie = tmp_dir.manage_cutObjects([new_id])
        themes_container.manage_pasteObjects(cookie)
        themes_container.manage_delObjects([tmp_dir.getId(),])
      
        # XXX to finish
        #new_theme =  themes_container[new_id]
        #catalog = getToolByName(self, 'portal_catalog')
        #searchpath = '/' + new_theme.absolute_url(relative=1)
        #res = catalog.searchResults(path=searchpath)

        #list_to_del = []
        #for brain in res:
        #  if brain is not None:
        #     ok_type = 0
        #     o =  brain.getObject()
        #     if o is not None:
        #        for allowed_type in ['isPortalTemplet', 'isPortalStyle', 'isCellModifier', 'isPortalPageBlock', 'isPortalPalette', 'isThemeFolder', 'isPortalStyle' ]:
        #           meth = getattr(o.aq_explicit, allowed_type, None) 
        #           if callable(meth) and meth():
        #               ok_type = 1
        #               continue
                #fti = o.getTypeInfo() 
                #if fti is not None:
                #   meta_type = fti.Title()
                #   if meta_type in ['Portal Image']:
                #      continue

        #        if ok_type == 0:
        #           list_to_del.append(o.absolute_url(relative=1))
 
        # new_theme.rebuild() 
        
        return new_id


    security.declareProtected('Manage Themes', 'manage_rebuild')
    def manage_rebuild(self, REQUEST=None):
        """
        Rebuild this theme
        """

        self.rebuild()
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')


    security.declareProtected('Manage Themes', 'rebuild')
    def rebuild(self):
        """
        Rebuild all the themes
        """

        for theme in self.objectValues():
            theme.rebuild()



InitializeClass(PortalThemesTool)


